/****************************************************
**
** PICProgrammerHardwareSerial.CPP
**
****************************************************/

//#include "PICProgrammerHardwareSerial.H"
#include "PICPHS.H"
#include "PICPT.H"
#include <assert.h>
#include <conio.h>
#include <stdio.h>

#define	COM_PORT_TABLE		((unsigned short *)0x00000400)

/*
**
*/
PICProgrammerHardwareSerialClass::PICProgrammerHardwareSerialClass( int com_port ) :
	PortRegister( 0 ),
	COMPort( 0 )
{
	if ( com_port < 1 ) {	// search for the first lpt port with a programmer
		for ( COMPort = 1; COMPort <= 4; COMPort++ ) {
			if ( Set_COM_Port( COMPort ) == NO_ERROR ) {
				break;
			}
		}
	} else {
		COMPort = com_port;
		Set_COM_Port( COMPort );
	}
}

/*
**
*/
PICProgrammerHardwareSerialClass::~PICProgrammerHardwareSerialClass( void )
{
	Set_State( VDD_OFF | VPP_OFF | CLOCK_OFF | DATA_OFF );
}

/*
**
*/
int	PICProgrammerHardwareSerialClass::Found( void )
{
	return (PortRegister != 0);
}

/*
**
*/
char * PICProgrammerHardwareSerialClass::Get_Description( void )
{
	static char Description[12];
	sprintf( Description, "COM%d 0x%3X", COMPort, PortRegister );
	return Description;
}

/*
**
*/
int PICProgrammerHardwareSerialClass::Set_COM_Port( int port )
{
	PortRegister = COM_PORT_TABLE[ port-1 ];

	if ( (PortRegister != 0x3F8) && (PortRegister != 0x3E8) && 
		  (PortRegister != 0x2F8) && (PortRegister != 0x2E8) ) {
		PortRegister = 0;
		return PORT_NOT_FOUND_ERROR;
	}

//	Delay_Ms( 0 );								// dummy call to setup CTC

//	Set_State( DATA_ON );
	Set_State( DATA_ON + VPP_ON );
	Delay_Ms( 100 );
	if ( Get_Input() == 0 ) {
		PortRegister = 0;
		return PROGRAMMER_NOT_FOUND_ERROR;
	}
//	Set_State( DATA_OFF );
	Set_State( DATA_OFF + VPP_ON );
	Delay_Ms( 100 );
	if ( Get_Input() != 0 ) {
		PortRegister = 0;
		return PROGRAMMER_NOT_FOUND_ERROR;
	}

	return NO_ERROR;
}

/*
**
*/
void PICProgrammerHardwareSerialClass::Set_State( int state_bits )
{
	assert( PortRegister != 0 );

	int port_bits;
	inportb( PortRegister + 3 );
	inportb( PortRegister + 3 );
	port_bits = inportb( PortRegister + 3 );
	if ( state_bits & VPP_ON )				port_bits |= 64;
	if ( state_bits & VPP_OFF )			port_bits &= ~64;
	outportb( PortRegister + 3, port_bits );

	inportb( PortRegister + 4 );
	inportb( PortRegister + 4 );
	port_bits = inportb( PortRegister + 4 );
//	if ( state_bits & VDD_ON )				port_bits |= 4;
//	if ( state_bits & VDD_OFF )			port_bits &= ~4;
	if ( state_bits & CLOCK_ON )			port_bits |= 2;
	if ( state_bits & CLOCK_OFF )			port_bits &= ~2;
	if ( state_bits & DATA_ON )			port_bits |= 1;
	if ( state_bits & DATA_OFF )			port_bits &= ~1;
	outportb( PortRegister + 4, port_bits );
}

/*
**
*/
int PICProgrammerHardwareSerialClass::Get_Input( void )
{
	assert( PortRegister != 0 );
	inportb( PortRegister + 6 );
	inportb( PortRegister + 6 );
	return (inportb( PortRegister + 6) & 0x10) ? 1 : 0;
}

